function cvx_optpnt = ando_epi_cone(sz1,sz2,t,iscplx)

%ANDO_EPI_CONE    Cone associated to Ando's convexity theorem.
%   ando_epi_cone(sz1,sz2,t,iscplx) returns a CVX tuple {A,B,T} of matrices
%   constrained to satisfy 
%     A^{1-t} \otimes B^t <= T
%   where \otimes denotes Kronecker product and where t is a number
%   in [-1,0] or [1,2].
%   Parameter sz1 and sz2 correspond to the sizes of matrices A and B
%   respectively. T automatically has size sz1*sz2.
%   If iscplx=1, matrices A,B,T are Hermitian, otherwise they are real
%   symmetric.
%
%AUTHORS
%   Hamza Fawzi (hfawzi@mit.edu) and James Saunderson (jamesfs@uw.edu)
%
%REFERENCE
%   This code is based on the paper: "Lieb's concavity theorem, matrix
%   geometric means and semidefinite optimization" by Hamza Fawzi and James
%   Saunderson (arXiv:1512.03401)

if nargin < 3
    error('Not enough input arguments');
end

if nargin < 4
    iscplx = 0;
end

if t < -1 || (t > 0 && t < 1) || t > 2
    error('t has to be in [-1,0] \cup [1,2]');
end

cvx_begin set
    if iscplx
        variable A(sz1,sz1) hermitian
        variable B(sz2,sz2) hermitian
        variable T(sz1*sz2,sz1*sz2) hermitian
    else
        variable A(sz1,sz1) symmetric
        variable B(sz2,sz2) symmetric
        variable T(sz1*sz2,sz1*sz2) symmetric
    end
    {kron(A,eye(sz2)),kron(eye(sz1),B),T} == ...
        matrix_geo_mean_epi_cone(sz1*sz2,t,iscplx);
cvx_end

cvx_optpnt = cvxtuple(struct('A',A,'B',B,'T',T));

end