function cvx_optpnt = matrix_geo_mean_epi_cone(sz,t,iscplx)

%MATRIX_GEO_MEAN_EPI_CONE    Matrix geometric mean cone.
%   matrix_geo_mean_epi_cone(sz,t,iscplx) returns a CVX tuple {A,B,T} of
%   matrices constrained to satisfy A #_{t} B <= T where A #_{t} B is the
%   t-weighted geometric mean of A and B. Parameter t should be a number in
%   [-1,0] or [1,2].
%   Parameter sz corresponds to the size of matrices A,B,T.
%   If iscplx=1, matrices A,B,T are Hermitian, otherwise they are real
%   symmetric.
%
%   For example writing {A,B,T}  == matrix_geo_mean_hypo_cone(4,3/2,0)
%   enforces the constraint A #_{3/2} B <= T, where A,B,T are 4x4 real
%   symmetric matrices.
%
%AUTHORS
%   Hamza Fawzi (hfawzi@mit.edu) and James Saunderson (jamesfs@uw.edu)
%
%REFERENCE
%   This code is based on the paper: "Lieb's concavity theorem, matrix
%   geometric means and semidefinite optimization" by Hamza Fawzi and James
%   Saunderson (arXiv:1512.03401)

narginchk(2,3);

if t < -1 || (t > 0 && t < 1) || t > 2
    error('t has to be in [-1,0] or [1,2]');
end

if length(sz) == 1
    sz = [sz sz];
end
if size(sz,2) == 1
    sz = sz';
end
dsz = [2*sz(1) 2*sz(2) sz(3:end)];

cvx_begin set
    if iscplx
        variable A(sz) hermitian
        variable B(sz) hermitian
        variable T(sz) hermitian
        variable Z(sz) hermitian
    else
        variable A(sz) symmetric
        variable B(sz) symmetric
        variable T(sz) symmetric
        variable Z(sz) symmetric
    end
    if t <= 0
        [T A; A Z] == semidefinite(dsz,iscplx);
        {A,B,Z} == matrix_geo_mean_hypo_cone(sz,-t,iscplx);
    elseif t >= 1
        [T B; B Z] == semidefinite(dsz,iscplx);
        {A,B,Z} == matrix_geo_mean_hypo_cone(sz,2-t,iscplx);
    end
cvx_end

cvx_optpnt = cvxtuple(struct('A',A,'B',B,'T',T));

end