function cvx_optpnt = matrix_geo_mean_hypo_cone(sz,t,iscplx)

%MATRIX_GEO_MEAN_HYPO_CONE    Matrix geometric mean cone.
%   matrix_geo_mean_hypo_cone(sz,t,iscplx) returns a CVX tuple {A,B,T} of
%   matrices constrained to satisfy A #_{t} B >= T where A #_{t} B is the
%   t-weighted geometric mean of A and B. Parameter t should be a number
%   between 0 and 1.
%   Parameter sz corresponds to the size of matrices A,B,T.
%   If iscplx=1, matrices A,B,T are Hermitian, otherwise they are real
%   symmetric.
%
%   For example writing {A,B,T} == matrix_geo_mean_hypo_cone(4,1/8,0)
%   enforces the constraint A #_{1/8} B >= T, where A,B,T are 4x4 real
%   symmetric matrices.
%
%AUTHORS
%   Hamza Fawzi (hfawzi@mit.edu) and James Saunderson (jamesfs@uw.edu)
%
%REFERENCE
%   This code is based on the paper: "Lieb's concavity theorem, matrix
%   geometric means and semidefinite optimization" by Hamza Fawzi and James
%   Saunderson (arXiv:1512.03401)

if nargin < 1
    error('Not enough input arguments');
end

if nargin < 2
    t = 1/2;
end

if nargin < 3
    iscplx = 0;
end

if t < 0 || t > 1
    error('t has to be between 0 and 1');
end

if length(sz) == 1
    sz = [sz sz];
end
if size(sz,2) == 1
    sz = sz';
end
dsz = [2*sz(1) 2*sz(2) sz(3:end)];

[p,q] = rat(t);
isPowerOfTwo = @(k) ~bitand(k,k-1);

cvx_begin set
    if iscplx
        variable A(sz) hermitian
        variable B(sz) hermitian
        variable T(sz) hermitian
    else
        variable A(sz) symmetric
        variable B(sz) symmetric
        variable T(sz) symmetric
    end
    if t == 0
        A == semidefinite(sz,iscplx);
        B == semidefinite(sz,iscplx);
        A - T == semidefinite(sz,iscplx);
    elseif t == 1
        A == semidefinite(sz,iscplx);
        B == semidefinite(sz,iscplx);
        B - T == semidefinite(sz,iscplx);
    elseif t == 1/2
        if iscplx
            variable W(sz) hermitian
        else
            variable W(sz) symmetric
        end
        [A W; W B] == semidefinite(dsz,iscplx);
        W - T == semidefinite(sz,iscplx);
    elseif isPowerOfTwo(q)
        % Dyadic number
        if iscplx
            variable Z(sz) hermitian
        else
            variable Z(sz) symmetric
        end
        if t < 1/2
            [A T; T Z] == semidefinite(dsz,iscplx);
            {A,B,Z} == matrix_geo_mean_hypo_cone(sz,2*t,iscplx);
        else
            [B T; T Z] == semidefinite(dsz,iscplx);
            {A,B,Z} == matrix_geo_mean_hypo_cone(sz,2*t-1,iscplx);
        end
    elseif isPowerOfTwo(p) && t > 1/2
        % Numerator is a power of two and t > 1/2
        if iscplx
            variable Z(sz) hermitian
            variable W(sz) hermitian
        else
            variable Z(sz) symmetric
            variable W(sz) symmetric
        end
        {A,W,Z} == matrix_geo_mean_hypo_cone(sz,(2*p-q)/p,iscplx);
        [Z W; W B] == semidefinite(dsz,iscplx);
        W - T == semidefinite(sz,iscplx);
    elseif t < 1/2
        % General case t < 1/2
        % Decompose t in t = (p/2^l) * (2^l/q) where l=floor(log2(q))
        l = floor(log2(q));
        if iscplx
            variable X(sz) hermitian
        else
            variable X(sz) symmetric
        end
        {A,B,X} == matrix_geo_mean_hypo_cone(sz,p/(2^l),iscplx);
        {A,X,T} == matrix_geo_mean_hypo_cone(sz,(2^l)/q,iscplx);
    else
        % General case t \in [1/2,1]
        % Use transformation t <-> 1-t to bring it to case t \in [0,1/2]
        {B,A,T} == matrix_geo_mean_hypo_cone(sz,1-t,iscplx);
    end
cvx_end

cvx_optpnt = cvxtuple(struct('A',A,'B',B,'T',T));

end