function cvx_optval = trace_KAK(A,K,t)

%TRACE_KAK    Implementation for trace[(K'*A^t*K)^{1/t}].
%   For a positive semidefinite matrix A of size nxn and a matrix K of size
%   nxm, trace_KAK(A,K,t) returns the value trace[(K'*A^t*K)^{1/t}]. This
%   function is concave (in A) when t is in [-1,1]\{0} and convex when t is
%   in [1,2].
%
%AUTHORS
%   Hamza Fawzi (hfawzi@mit.edu) and James Saunderson (jamesfs@uw.edu)
%
%REFERENCE
%   This code is based on the paper: "Lieb's concavity theorem, matrix
%   geometric means and semidefinite optimization" by Hamza Fawzi and James
%   Saunderson (arXiv:1512.03401)

if nargin < 3
    error('Not enough input arguments');
end

if ~ismatrix(A) || size(A,1) ~= size(A,2)
    error('A must be a square matrix');
end
if ~ismatrix(K) || size(K,1) ~= size(A,2)
    error('B must be a matrix where the number of rows is equal to the number of columns as A');
end

if t == 0
    error('t should be nonzero');
end

if isnumeric(A)
    cvx_optval = real(trace(mpower(K'*mpower(A,t)*K,1/t)));
elseif cvx_isaffine(A)
    if t == 1
        cvx_optval = trace(K'*A*K);
        return;
    end
    n = size(A,1);
    m = size(K,2);
    iscplx = ~isreal(A) || ~isreal(K);
    cvx_begin
        if iscplx
            variable X(m,m) hermitian
        else
            variable X(m,m) symmetric
        end
        if t >= 1 && t <= 2
            minimize ((1/t)*(lieb_ando(A,X,K,1-t) + (t-1)*trace(X)));
        elseif -1 <= t && t <= 1
            maximize ((1/t)*(lieb_ando(A,X,K,1-t) + (t-1)*trace(X)));
        else
            error('t must be between -1 and 2');
        end
        X == semidefinite(m,iscplx);
    cvx_end
else
    error('Disciplined convex programming error:\n The input has to be an affine expression');
end