function cvx_optval = trace_mpower(A,t,C)

%TRACE_MPOWER    Trace of matrix power
%   For positive semidefinite matrix A, trace_mpower(A,t) returns
%   trace(A^t). This is a concave function in A if t \in [0,1], and a
%   convex function in A if t \in [-1,0] \cup [1,2].
%   The function can also accept a third parameter C, in which case
%   trace_mpower(A,t,C) returns the value trace(C*A^t). C must be a
%   positive semidefinite matrix (otherwise the function is not necessarily
%   convex/concave).
%
%AUTHORS
%   Hamza Fawzi (hfawzi@mit.edu) and James Saunderson (jamesfs@uw.edu)
%
%REFERENCE
%   This code is based on the paper: "Lieb's concavity theorem, matrix
%   geometric means and semidefinite optimization" by Hamza Fawzi and James
%   Saunderson (arXiv:1512.03401)

if nargin < 2
    error('Not enough input arguments');
end
if ~ismatrix(A) || size(A,1) ~= size(A,2)
    error('X must be a square matrix');
end
if nargin < 3
    C = eye(size(A,1));
else
    % Check that C is positive semidefinite
    C = (C+C')/2;
    if any(eig(C) < -1e-6)
        error('C has to be positive semidefinite');
    end
end

if isnumeric(A)
    cvx_optval = trace(C*mpower(A,t));
elseif cvx_isaffine(A)
    if t < -1 || t > 2
        error('t must be between -1 and 2');
    end
    n = size(A,1);
    iscplx = ~isreal(A);
    cvx_begin
        if iscplx
            variable T(n,n) hermitian
        else
            variable T(n,n) symmetric
        end
        if t >= 0 && t <= 1
            % Concave function
            maximize trace(C*T)
            {eye(n),A,T} == matrix_geo_mean_hypo_cone(n,t,iscplx);
        else
            % Convex function
            minimize trace(C*T)
            {eye(n),A,T} == matrix_geo_mean_epi_cone(n,t,iscplx);
        end
    cvx_end    
else
    error('Disciplined convex programming error:\n The input has to be an affine expression');
end

end