function cvx_optval = tsallis_entr(A,t)

%TSALLIS_ENTR    Tsallis entropy
%   For a positive semidefinite matrix A, and t in [0,1], tsallis_entr(A,t)
%   returns the value trace[A^{1-t}-A]/t. This function is concave in A for
%   any t in [0,1]. When t->0 this function approaches the von Neumann
%   entropy S(A) = -trace[A*logm(A)].
%   To approximate von Neumann entropy using this function, use a small
%   power of two, e.g., t = 2^(-5) or 2^(-6). Note that very small values
%   can lead to numerically ill-conditioned problems.
%
%AUTHORS
%   Hamza Fawzi (hfawzi@mit.edu) and James Saunderson (jamesfs@uw.edu)
%
%REFERENCE
%   This code is based on the paper: "Lieb's concavity theorem, matrix
%   geometric means and semidefinite optimization" by Hamza Fawzi and James
%   Saunderson (arXiv:1512.03401)


% Check input arguments
if nargin < 2
    error('Not enough input arguments');
end

if size(A,1) ~= size(A,2)
    error('Input must be a square matrix.');
end

if t < 0 || t > 1
    error('r should be between 0 and 1');
end

if isnumeric(A)
    cvx_optval = real(trace(mpower(A,1-t) - A)/t);
elseif cvx_isaffine(A)
    n = size(A,1);
    iscplx = ~isreal(A);
    cvx_begin
        if iscplx
            variable T(n,n) hermitian
        else
            variable T(n,n) symmetric
        end
        maximize ((1/t)*trace(T-A));
        {A,eye(n),T} == matrix_geo_mean_hypo_cone(n,t,iscplx);
    cvx_end

else
    error('Disciplined convex programming error:\n The input has to be an affine expression');
end

    
end