function cvx_optval = tsallis_rel_entr(A,B,t)

%TSALLIS_REL_ENTR    Tsallis relative entropy
%   For positive semidefinite matrices A and B of the same size and t
%   in [0,1], tsallis_rel_entr(A,B,t) returns the value
%   trace[A - A^{1-t}*B^t]/t. This is a convex function in (A,B) when t is
%   in [0,1]. When t->0 this function approaches the quantum relative
%   entropy S(A|B) = trace[A*(logm(A)-logm(B))].
%   To approximate quantum relataive entropy using this function, use a
%   small power of two, e.g., t = 2^(-5) or 2^(-6). Note that very small
%   values can lead to numerically ill-conditioned problems.
%
%AUTHORS
%   Hamza Fawzi (hfawzi@mit.edu) and James Saunderson (jamesfs@uw.edu)
%
%REFERENCE
%   This code is based on the paper: "Lieb's concavity theorem, matrix
%   geometric means and semidefinite optimization" by Hamza Fawzi and James
%   Saunderson (arXiv:1512.03401)


if nargin < 3
    error('Not enough input arguments');
end

if t < 0 || t > 1
    error('t should be between 0 and 1');
end

if ~ismatrix(A) || ~ismatrix(B) || size(A,1) ~= size(A,2) ...
        || size(B,1) ~= size(B,2) || size(A,1) ~= size(B,1)
    error('A and B must be square matrices of the same size');
end

n = size(A,1);

cvx_optval = (trace(A) - lieb_ando(A,B,eye(n),t))/t;

end